@@ -95,7 +95,7 @@ initgrav(win)
   gravstruct *gp = &gravs[screen];
   unsigned char ball;
   
-  XGetWindowAttributes(dsp, win, &xgwa);
+  (void) XGetWindowAttributes(dsp, win, &xgwa);
   gp->width = xgwa.width;
   gp->height = xgwa.height;
  
@@ -111,11 +111,9 @@ initgrav(win)
     gp->planets = (planetstruct *) calloc(gp->nplanets, sizeof(planetstruct));
   XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
   XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, gp->width, gp->height);
-  if (!mono && Scr[screen].npixels > 2) {
-    gp->starcolor = random() % Scr[screen].npixels;
-    if (gp->starcolor == BlackPixel(dsp, screen))
-      gp->starcolor = WhitePixel(dsp, screen);
-  } else
+  if (!mono && Scr[screen].npixels > 2)
+    gp->starcolor = Scr[screen].pixels[random() % Scr[screen].npixels];
+  else
     gp->starcolor = WhitePixel(dsp, screen);
   for (ball = 0; ball < gp->nplanets; ball++)
     init_planet(win, &gp->planets[ball]);
@@ -133,11 +131,9 @@ init_planet(win, planet)
 {
   gravstruct *gp = &gravs[screen];
 
-    if (!mono && Scr[screen].npixels > 2) {
-      planet->colors = random() % Scr[screen].npixels;
-      if (planet->colors == BlackPixel(dsp, screen))
-        planet->colors = WhitePixel(dsp, screen);
-    } else
+    if (!mono && Scr[screen].npixels > 2)
+      planet->colors = Scr[screen].pixels[random() % Scr[screen].npixels];
+    else
       planet->colors = WhitePixel(dsp, screen);
     /* Initialize positions */
     POS(X) = FLOATRAND(-XR,XR);
