@@ -11,15 +11,19 @@ static char sccsid[] = "@(#)resource.c	1.20 91/09/27 XLOCK";
  * Revision History:
  *
  * Changes of David Bagley (bagleyd@source.asset.com)
- * 20-Sep-94: added bat mode from Lorenzo Patocchi <patol@info.isbiel.ch>
+ * 21-Dec-94: patch for -delay, -batchcount and -saturation for X11R5+
+ *            from Patrick D Sullivan <pds@bss.com>. 
+ * 18-Dec-94: -inroot option added from Bill Woodward <wpwood@pencom.com>.
+ * 31-Oct-94: bomb mode was added from <D.T.Shield@compsci.liverpool.ac.uk>.
+ * 20-Sep-94: added bat mode from Lorenzo Patocchi <patol@info.isbiel.ch>.
  * 11-Jul-94: added grav mode, and inwindow option from Greg Bowering
  *            <greg@cs.adelaide.edu.au>
  * 22-Jun-94: Modified for VMS
  *            <Anthony.D.Clarke@Support.Hatfield.Raytheon.bae.eurokom.ie>
  * 17-Jun-94: default changed from life to blank
- * 21-Mar-94: patch fix for AIXV3 from (R.K.Lloyd@csc.liv.ac.uk)
- * 01-Dec-93: added patch for AIXV3 from
- *            (Tom McConnell, tmcconne@sedona.intel.com)
+ * 21-Mar-94: patch fix for AIXV3 from <R.K.Lloyd@csc.liv.ac.uk>
+ * 01-Dec-93: added patch for AIXV3 from Tom McConnell
+ *            <tmcconne@sedona.intel.com>
  * 29-Jun-93: added spline, maze, sphere, hyper, helix, rock, and blot mode.
  *
  * Changes of Patrick J. Naughton
@@ -38,8 +42,8 @@ static char sccsid[] = "@(#)resource.c	1.20 91/09/27 XLOCK";
  *
  */
 
-#include <stdio.h>
 #include "xlock.h"
+#include <stdio.h>
 #include <netdb.h>
 #include <math.h>
 #include <ctype.h>
@@ -58,9 +62,6 @@ extern void drawlife();
 extern void initqix();
 extern void drawqix();
 
-extern void initblank();
-extern void drawblank();
-
 extern void initswarm();
 extern void drawswarm();
 
@@ -108,7 +109,7 @@ extern void drawworld();
 
 extern void initrect();
 extern void drawrect();
-
+ 
 extern void initbat();
 extern void drawbat();
 
@@ -118,6 +119,28 @@ extern void drawgalaxy();
 extern void initkaleid();
 extern void drawkaleid();
 
+extern void initwator();
+extern void drawwator();
+
+extern void initlife3d();
+extern void drawlife3d();
+
+extern void initswirl();
+extern void drawswirl();
+
+extern void initimage();
+extern void drawimage();
+
+extern void initbomb();
+extern void drawbomb();
+
+extern void initblank();
+extern void drawblank();
+
+#ifdef FORCE_BOMB
+extern int full_lock();
+#endif
+
 typedef struct {
     char       *cmdline_arg;
     void        (*lp_init) ();
@@ -128,14 +151,17 @@ typedef struct {
     char       *desc;
 }           LockStruct;
 
+#define NUMSPECIAL 4 /* # of Special Screens */
+static char swirlstring[] = "swirl";
 static char randomstring[] = "random";
+static char bombstring[] = "bomb";
 
 static LockStruct LockProcs[] = {
     {"hop", inithop, drawhop, 1000, 1000, 1.0, "Hopalong iterated fractals"},
     {"qix", initqix, drawqix, 30000, 64, 1.0, "Spinning lines a la Qix(tm)"},
-    {"life", initlife, drawlife, 1000000, 100, 1.0, "Conway's game of Life"},
+    {"life", initlife, drawlife, 750000, 125, 1.0, "Conway's game of Life"},
     {"swarm", initswarm, drawswarm, 10000, 100, 1.0, "Swarm of bees"},
-    {"rotor", initrotor, drawrotor, 100000, 4, 0.4, "Tom's Roto-Rooter"},
+    {"rotor", initrotor, drawrotor, 10000, 4, 0.4, "Tom's Roto-Rooter"},
     {"pyro", initpyro, drawpyro, 15000, 40, 1.0, "Fireworks"},
     {"flame", initflame, drawflame, 10000, 20, 1.0, "Cosmic Flame Fractals"},
     {"worm", initworm, drawworm, 10000, 20, 1.0, "Wiggly Worms"},
@@ -144,16 +170,21 @@ static LockStruct LockProcs[] = {
     {"sphere", initsphere, drawsphere, 10000, 1, 1.0, "Shaded spheres"},
     {"hyper", inithyper, drawhyper, 10000, 1, 1.0, "Spinning Tesseract"},
     {"helix", inithelix, drawhelix, 10000, 1, 1.0, "Helix"},
-    {"rock", initrock, drawrock, 30000, 100, 1.0, "Asteroid field"},
+    {"rock", initrock, drawrock, 20000, 100, 1.0, "Asteroid field"},
     {"blot", initblot, drawblot, 10000, 6, 0.4, "Rorschach's ink blot test"},
     {"grav", initgrav, drawgrav, 10000, 10, 1.0, "Orbiting planets"},
     {"bounce", initbounce, drawbounce, 10000, 10, 1.0, "Bouncing ball"},
     {"world", initworld, drawworld, 100000, 8, 0.3, "Random Spinning Earths"},
     {"rect", initrect, drawrect, 10000, 100, 1.0, "Greynetic rectangles"},
-    {"bat", initbat, drawbat, 10000, 6, 1.0, "Flying Bats"},
+    {"bat", initbat, drawbat, 100000, 6, 1.0, "Flying Bats"},
     {"galaxy", initgalaxy, drawgalaxy, 100, 3, 1.0, "Spinning galaxies"},
     {"kaleid", initkaleid, drawkaleid, 2000, 4, 1.0, "Kaleidoscope"},
-    {"blank", initblank, drawblank, 5000000, 1, 1.0, "Blank screen"},
+    {"wator", initwator, drawwator, 750000, 4, 1.0, "Dewdney's Planet Wa-Tor"},
+    {"life3d", initlife3d, drawlife3d, 1000000, 40, 1.0, "Bayes' 3D Life"},
+    {swirlstring, initswirl, drawswirl, 5000, 5, 1.0, "Animated swirling patterns"},
+    {"image", initimage, drawimage, 2000000, 1, 1.0, "Random Bouncing Image"},
+    {bombstring, initbomb, drawbomb, 1000000, 1, 1.0, "Temporary screen lock"},
+    {"blank", initblank, drawblank, 4000000, 1, 1.0, "Blank screen"},
     {randomstring, NULL, NULL, 0, 0, 0.0, "Random mode"},
 };
 
@@ -223,6 +254,8 @@ static XrmOptionDescRec genTable[] = {
     {"+v", ".verbose", XrmoptionNoArg, (caddr_t) "off"},
     {"-inwindow",".inwindow", XrmoptionNoArg, (caddr_t) "on"},
     {"+inwindow",".inwindow", XrmoptionNoArg, (caddr_t) "off"},
+    {"-inroot",".inroot", XrmoptionNoArg, (caddr_t) "on"},
+    {"+inroot",".inroot", XrmoptionNoArg, (caddr_t) "off"},
     {"-timeelapsed", ".timeelapsed", XrmoptionNoArg, (caddr_t) "on"},
     {"+timeelapsed", ".timeelapsed", XrmoptionNoArg, (caddr_t) "off"},
     {"-nice", ".nice", XrmoptionSepArg, (caddr_t) NULL},
@@ -240,10 +273,11 @@ static XrmOptionDescRec genTable[] = {
 };
 #define genEntries (sizeof genTable / sizeof genTable[0])
 
+char Delay[64], BatchCount[64], Saturation[64];
 static XrmOptionDescRec modeTable[] = {
-    {"-delay", ".delay", XrmoptionSepArg, (caddr_t) NULL},
-    {"-batchcount", ".batchcount", XrmoptionSepArg, (caddr_t) NULL},
-    {"-saturation", ".saturation", XrmoptionSepArg, (caddr_t) NULL},
+    {"-delay", Delay, XrmoptionSepArg, (caddr_t) NULL},
+    {"-batchcount", BatchCount, XrmoptionSepArg, (caddr_t) NULL},
+    {"-saturation", Saturation, XrmoptionSepArg, (caddr_t) NULL},
 };
 #define modeEntries (sizeof modeTable / sizeof modeTable[0])
 
@@ -253,6 +287,10 @@ static XrmOptionDescRec cmdlineTable[] = {
     {"+nolock", ".nolock", XrmoptionNoArg, (caddr_t) "off"},
     {"-remote", ".remote", XrmoptionNoArg, (caddr_t) "on"},
     {"+remote", ".remote", XrmoptionNoArg, (caddr_t) "off"},
+    {"-inwindow", ".inwindow", XrmoptionNoArg, (caddr_t) "on"},
+    {"+inwindow", ".inwindow", XrmoptionNoArg, (caddr_t) "off"},
+    {"-inroot", ".inroot", XrmoptionNoArg, (caddr_t) "on"},
+    {"+inroot", ".inroot", XrmoptionNoArg, (caddr_t) "off"},
 };
 #define cmdlineEntries (sizeof cmdlineTable / sizeof cmdlineTable[0])
 
@@ -281,6 +319,7 @@ static OptionStruct opDesc[] = {
     {"-/+usefirst", "turn on/off using the first char typed in password"},
     {"-/+v", "turn on/off verbose mode"},
     {"-/+inwindow", "turn on/off making xlock run in a window"},
+    {"-/+inroot", "turn on/off making xlock run in the root window"},
     {"-/+timeelapsed", "turn on/off clock"},
     {"-delay usecs", "microsecond delay between screen updates"},
     {"-batchcount num", "number of things per batch"},
@@ -323,6 +362,7 @@ Bool        echokeys;
 Bool        usefirst;
 Bool        verbose;
 Bool        inwindow;
+Bool        inroot;
 Bool        timeelapsed;
 
 #define t_String	0
@@ -359,6 +399,7 @@ static argtype genvars[] = {
     {(caddr_t *) &usefirst, "usefirst", "Usefirst", "off", t_Bool},
     {(caddr_t *) &verbose, "verbose", "Verbose", "off", t_Bool},
     {(caddr_t *) &inwindow, "inwindow", "InWindow", "off", t_Bool},
+    {(caddr_t *) &inroot, "inroot", "InRoot", "off", t_Bool},
     {(caddr_t *) &timeelapsed, "timeelapsed", "TimeElapsed", "off", t_Bool},
 };
 #define NGENARGS (sizeof genvars / sizeof genvars[0])
@@ -618,8 +659,7 @@ static void
 open_display()
 {
     if (displayname != NULL) {
-	extern char *strchr();
-	char       *colon = strchr(displayname, ':');
+	char       *colon = (char *) strchr(displayname, ':');
 	int         n = colon - displayname;
 
 	if (colon == NULL)
@@ -773,6 +813,12 @@ GetResources(argc, argv)
 		"off", (caddr_t *) &nolock);
     GetResource(RDB, ProgramName, classname, "remote", "Remote", t_Bool,
 		"off", (caddr_t *) &remote);
+    GetResource(RDB, ProgramName, classname, "inwindow", "InWindow", t_Bool,
+		"off", (caddr_t *) &inwindow);
+    GetResource(RDB, ProgramName, classname, "inroot", "InRoot", t_Bool,
+		"off", (caddr_t *) &inroot);
+    GetResource(RDB, ProgramName, classname, "mono", "Mono", t_Bool,
+		"off", (caddr_t *) &mono);
 
     open_display();
     serverString = XResourceManagerString(dsp);
@@ -794,14 +840,35 @@ GetResources(argc, argv)
 
     /*
      * if random< mode, then just grab a random entry from the table
+     * or root is trying to run bomb ... sorry not a good idea,
+     * attempting to run swirl in root window is not a good idea either
      */
+   
     if (!strcmp(mode, randomstring))
-	mode = LockProcs[random() % (NUMPROCS - 2)].cmdline_arg;
+      mode = LockProcs[random() % (NUMPROCS - NUMSPECIAL)].cmdline_arg;
+    if (!getuid() && !strcmp(mode, bombstring))
+       error("%s: mode %s can not run as root.\n", bombstring);
+#ifndef FORCEFIXEDCOLORS
+    if (!mono && inroot && !strcmp(mode, swirlstring))
+       error("%s: mode %s can not run in root window.\n", swirlstring);
+#endif
+#ifdef FORCE_BOMB
+    /*
+     *  Force temporary locking mode
+     */
+    if ( !full_lock() )
+      mode = LockProcs[(NUMPROCS - NUMSPECIAL + 1)].cmdline_arg;
+#endif
 
     (void) sprintf(modename, "%s.%s", ProgramName, mode);
     (void) sprintf(modeclass, "%s.%s", classname, mode);
 
-    XrmParseCommand(&modeDB, modeTable, modeEntries, modeclass, &argc, argv);
+    /* X11R5 may need these changes to make the modeTable stuff work */
+    for (i = 0; i < NMODEARGS; i++)
+      (void) sprintf(modeTable[i].specifier, ".%s.%s", mode, modevars[i].name);
+
+    XrmParseCommand(&modeDB, modeTable, modeEntries, ProgramName, &argc, argv);
+    /*XrmParseCommand(&modeDB, modeTable, modeEntries, modeclass, &argc, argv);*/
     (void) XrmMergeDatabases(modeDB, &RDB);
 
     /* Parse the rest of the command line */
