@@ -71,11 +71,9 @@ rock_reset(win, arocks)
   arocks->r = (RESOLUTION * 0.7) + (random() % (30 * RESOLUTION));
   arocks->theta = random() % RESOLUTION;
   arocks->depth = MAX_DEPTH * DEPTH_SCALE;
-  if (!mono && Scr[screen].npixels > 2) {
-    arocks->color = random() % Scr[screen].npixels;
-    if (arocks->color == BlackPixel(dsp, screen))
-      arocks->color = WhitePixel(dsp, screen);
-  } else
+  if (!mono && Scr[screen].npixels > 2)
+    arocks->color = Scr[screen].pixels[random() % Scr[screen].npixels];
+  else
     arocks->color = WhitePixel(dsp, screen);
   rock_compute(arocks);
   rock_draw(win, arocks, True);
@@ -205,7 +203,7 @@ initrock(win)
   rockstruct *rp = &rocks[screen];
   int i;
   XWindowAttributes xgwa;
-  XGetWindowAttributes(dsp, win, &xgwa);
+  (void) XGetWindowAttributes(dsp, win, &xgwa);
 
       rp->width = xgwa.width;
       rp->height = xgwa.height;
