@@ -9,6 +9,8 @@ static char sccsid[] = "@(#)usleep.c	1.3 91/05/24 XLOCK";
  * Revision History:
  *
  * Changes of David Bagley <bagleyd@source.asset.com>
+ * 05-Jan-95: nanosleep for Solaris 2.3 and greater Greg Onufer
+ *            <Greg.Onufer@Eng.Sun.COM>
  * 22-Jun-94: Fudged for VMS by Anthony Clarke
  *            <Anthony.D.Clarke@Support.Hatfield.Raytheon.bae.eurokom.ie>
  * 10-Jun-94: patch for BSD from Victor Langeveld <vic@mbfys.kun.nl>
@@ -31,8 +33,17 @@ usleep(usec)
     unsigned long usec;
 {
 #if (defined (SYSV) || defined(SVR4)) && !defined(__hpux)
-    /*poll((struct poll *) 0, (size_t) 0, usec / 1000);*/ /* ms resolution */
-    poll((void *) 0, (size_t) 0, usec / 1000);	/* ms resolution */
+#if defined(HAS_NANOSLEEP)
+{
+    struct timespec rqt;
+
+    rqt.tv_sec = 0;
+    rqt.tv_nsec = 1000 * usec;
+    return nanosleep(&rqt, NULL);
+}
+#else
+    (void) poll((void *) 0, (size_t) 0, usec / 1000);	/* ms resolution */
+#endif
 #else
 #ifdef VMS
     long timadr[2];
