@@ -21,6 +21,8 @@ static char sccsid[] = "@(#)xlock.c	23.21 91/06/27 XLOCK";
  * Revision History:
  *
  * Changes of David Bagley <bagleyd@source.asset.com>
+ * 26-Oct-94: In order to use extra-long passwords with the Linux changed
+ *            PASSLENGTH to 64 <slouken@virtbrew.water.ca.gov>
  * 11-Jul-94: added -inwindow option from Greg Bowering
  *            <greg@cs.adelaide.edu.au>
  * 22-Jun-94: Modified for VMS
@@ -136,6 +138,9 @@ static char sccsid[] = "@(#)xlock.c	23.21 91/06/27 XLOCK";
 #include <stdio.h>
 #include <signal.h>
 #include <string.h>
+#ifdef __STDC__
+#include <stdarg.h>
+#endif
 #ifndef VMS
 #include <pwd.h>
 #endif
@@ -146,6 +151,7 @@ static char sccsid[] = "@(#)xlock.c	23.21 91/06/27 XLOCK";
 #include "xlock.h"
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
+#include "vroot.h"
 
 extern char *crypt();
 extern char *getenv();
@@ -180,7 +186,7 @@ static int  ssinterval;
 static int  ssblanking;
 static int  ssexposures;
 
-#define PASSLENGTH 20
+#define PASSLENGTH		64
 #define FALLBACK_FONTNAME	"fixed"
 #define ICONW			64
 #define ICONH			64
@@ -196,10 +202,22 @@ static int  ssexposures;
 
 
 /* VARARGS1 */
+#ifdef __STDC__
+void
+error(char *s1, ...)
+{
+    char    *s2;
+    va_list vl;
+
+    va_start(vl, s1);
+    s2 = va_arg(vl, char *);
+    va_end(vl);
+#else
 void
 error(s1, s2)
     char       *s1, *s2;
 {
+#endif
     (void) fprintf(stderr, s1, ProgramName, s2);
     exit(1);
 }
@@ -281,7 +299,7 @@ XChangeGrabbedCursor(cursor)
     Cursor      cursor;
 {
 #ifndef DEBUG
-    if (!inwindow)
+    if (!inwindow && !(inroot && nolock))
       (void) XGrabPointer(dsp, win[0], True, AllPointerEventMask,
 		    GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
 #endif
@@ -295,6 +313,8 @@ static void
 finish()
 {
     XSync(dsp, False);
+    if (!nolock && inroot)
+        XClearWindow(dsp, win[0]);
     if (!nolock && !allowaccess)
 	XUngrabHosts(dsp);
     XUngrabPointer(dsp, CurrentTime);
@@ -395,6 +415,9 @@ ReadXString(s, slen)
 	    }
 	    break;
 
+#ifdef MOUSEMOTION
+	case MotionNotify:
+#endif
 	case ButtonPress:
 	    if (((XButtonEvent *) & event)->window == icon[screen]) {
 		return 1;
@@ -415,7 +438,9 @@ ReadXString(s, slen)
 	case KeymapNotify:
 	case KeyRelease:
 	case ButtonRelease:
+#ifndef MOUSEMOTION
 	case MotionNotify:
+#endif
 	case LeaveNotify:
 	case EnterNotify:
 	case NoExpose:
@@ -440,8 +465,13 @@ getPassword()
 
     char        buffer[PASSLENGTH];
     char        userpass[PASSLENGTH];
+#ifdef SOLARIS_SHADOW
+    char      user[L_cuserid];
+    struct spwd *sp;
+#else
     char        rootpass[PASSLENGTH];
     char       *user;
+#endif
     XWindowAttributes xgwa;
     int         y, left, done;
 #ifdef VMS
@@ -516,51 +546,61 @@ getPassword()
     rootuser.addr  = root;
  
     sys$getuai(0,0,&rootuser,&il,0,0,0);
-#else
+#else /* !VMS */
 #ifdef OSF1_ENH_SEC
-  struct pr_passwd *pw;
+    struct pr_passwd *pw;
 
-  pw = getprpwnam("root");
-  strcpy(rootpass, pw->ufld.fd_encrypt);
+    pw = getprpwnam("root");
+    strcpy(rootpass, pw->ufld.fd_encrypt);
 
-  pw = getprpwuid (getuid());
-  strcpy(userpass, pw->ufld.fd_encrypt);
-  user = pw->ufld.fd_name;
+    pw = getprpwuid (getuid());
+    strcpy(userpass, pw->ufld.fd_encrypt);
+    user = pw->ufld.fd_name;
 
-#else
+#else /* !OSF1_ENH_SEC */
     struct passwd *pw;
 
-#ifdef __NetBSD__
-    seteuid(0);                       /* Need to be root now on NetBSD */
+#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD)
+    seteuid(0);       /* Need to be root now */
 #endif
+
+#ifdef SOLARIS_SHADOW
+    /* pw = (struct passwd *)getpwnam("root");
+       (void) strcpy(rootpass, pw->pw_passwd);
+       pw = (struct passwd *)getpwnam(cuserid(NULL));*/
+    sp = getspnam(cuserid(user));
+    strcpy(userpass, sp->sp_pwdp);
+#else
     pw = (struct passwd *)getpwnam("root");
     (void) strcpy(rootpass, pw->pw_passwd);
 
-#ifdef SOLARIS_SHADOW
-    pw = (struct passwd *)getpwnam(cuserid(NULL));
-#else
     pw = (struct passwd *)getpwuid(getuid());
+    /* Try this instead for HPUX */
+    /* pw = (struct passwd *)getpwnam(cuserid(NULL)); */
+
 #endif
 #if defined(HAS_SHADOW) && defined(linux)
-   {
-     struct spwd *spw;
+    {
+      struct spwd *spw;
 
-     if ((spw = getspnam(pw->pw_name)) == NULL)
-       pw->pw_passwd = "";
-     else
-       pw->pw_passwd = spw->sp_pwdp;
+      if ((spw = getspnam(pw->pw_name)) == NULL)
+        pw->pw_passwd = "";
+      else
+        pw->pw_passwd = spw->sp_pwdp;
     }
 #endif
+#ifndef SOLARIS_SHADOW
     (void) strcpy(userpass, pw->pw_passwd);
 
     user = pw->pw_name;
 #endif
-#ifdef __NetBSD__
+#endif /* !OFS_ENH_SEC */
+#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD)
     seteuid(getuid());                /* Don't need to be root anymore */
 #endif
-#endif
+#endif /* !VMS */
 
-    XGetWindowAttributes(dsp, win[screen], &xgwa);
+    (void) XGetWindowAttributes(dsp, win[screen], &xgwa);
 
     XChangeGrabbedCursor(XCreateFontCursor(dsp, XC_left_ptr));
 
@@ -598,15 +638,15 @@ getPassword()
 
     if (timeelapsed) {
       char clock[16];
-      int day, hr, min, sec;
+      int day, hour, minute, second;
       long time_displayed = seconds() - start_time;
       if (time_displayed < 0)
         time_displayed = 0;
       day = (int) (time_displayed / (24 * 60 * 60));
-      hr = (int) ((time_displayed / (24 * 60)) % 60);
-      min = (int) ((time_displayed / 60) % 60);
-      sec = (int) (time_displayed % 60);
-      (void) sprintf (clock, "%d:%02d:%02d:%02d", day, hr, min, sec);
+      hour = (int) ((time_displayed / (24 * 60)) % 60);
+      minute = (int) ((time_displayed / 60) % 60);
+      second = (int) (time_displayed % 60);
+      (void) sprintf (clock, "%d:%02d:%02d:%02d", day, hour, minute, second);
       y += font->ascent + font->descent + 2;
       XDrawString(dsp, win[screen], textgc[screen],
 		iconx[screen], y, clock, strlen(clock));
@@ -643,15 +683,22 @@ getPassword()
         root_system[8]=0;
         done = !((strcmp(hash_password,hash_system)) &&
                  (!allowroot || strcmp(root_password,root_system)));
-#else
+#else /* !VMS */
 #if defined(HAS_SHADOW) && defined(linux)
+#ifdef __QNX__
+#define pw_encrypt crypt
+#endif
         done = !((strcmp(pw_encrypt(buffer, userpass), userpass)) &&
           (!allowroot || strcmp(pw_encrypt(buffer, rootpass), rootpass)));
+#else
+#ifdef SOLARIS_SHADOW
+        done = !((strcmp(crypt(buffer, userpass), userpass)));
 #else
 	done = !((strcmp(crypt(buffer, userpass), userpass)) &&
           (!allowroot || strcmp(crypt(buffer, rootpass), rootpass)));
 #endif
 #endif
+#endif /* !VMS */
 	if (!done && !*buffer) {
 	    /* just hit return, and it wasn't his password */
 	    break;
@@ -719,7 +766,11 @@ justDisplay()
         if (!inwindow && event.type == VisibilityNotify)
 	    XRaiseWindow(dsp, event.xany.window);
 #endif
-    } while (event.type != ButtonPress && event.type != KeyPress);
+    } while (event.type != ButtonPress &&
+#ifdef MOUSEMOTION
+             event.type != MotionNotify &&
+#endif
+             event.type != KeyPress);
     for (screen = 0; screen < screens; screen++)
 	if (event.xbutton.root == RootWindow(dsp, screen))
 	    break;
@@ -744,12 +795,12 @@ lockDisplay()
 	sigset_t    oldsigmask;
 	sigset_t    newsigmask;
 
-	sigemptyset(&newsigmask);
-	sigaddset(&newsigmask, SIGHUP);
-	sigaddset(&newsigmask, SIGINT);
-	sigaddset(&newsigmask, SIGQUIT);
-	sigaddset(&newsigmask, SIGTERM);
-	sigprocmask(SIG_BLOCK, &newsigmask, &oldsigmask);
+	(void) sigemptyset(&newsigmask);
+	(void) sigaddset(&newsigmask, SIGHUP);
+	(void) sigaddset(&newsigmask, SIGINT);
+	(void) sigaddset(&newsigmask, SIGQUIT);
+	(void) sigaddset(&newsigmask, SIGTERM);
+	(void) sigprocmask(SIG_BLOCK, &newsigmask, &oldsigmask);
 #else
 	int         oldsigmask;
 #ifndef VMS
@@ -793,11 +844,11 @@ allocpixel(cmap, name, def)
 {
     XColor      col;
     XColor      tmp;
-    XParseColor(dsp, cmap, name, &col);
+    (void) XParseColor(dsp, cmap, name, &col);
     if (!XAllocColor(dsp, cmap, &col)) {
 	(void) fprintf(stderr, "couldn't allocate: %s, using %s instead\n",
 		 name, def);
-	XAllocNamedColor(dsp, cmap, def, &col, &tmp);
+	(void) XAllocNamedColor(dsp, cmap, def, &col, &tmp);
     }
     return col.pixel;
 }
@@ -822,7 +873,7 @@ main(argc, argv)
     set_auth_parameters();
 #endif
 
-#ifdef __NetBSD__
+#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD)
     seteuid(getuid());
 #endif
 
@@ -851,6 +902,8 @@ main(argc, argv)
 	root[screen] = RootWindowOfScreen(scr);
         bgcol[screen] = allocpixel(cmap, background, "White");
         fgcol[screen] = allocpixel(cmap, foreground, "Black");
+        Scr[screen].bgcol = bgcol[screen] ;
+        Scr[screen].fgcol = fgcol[screen] ;
 
 	if (mono || CellsOfScreen(scr) == 2) {
 	    Scr[screen].pixels[0] = fgcol[screen];
@@ -887,8 +940,11 @@ main(argc, argv)
 
 	xswa.override_redirect = True;
 	xswa.background_pixel = BlackPixelOfScreen(scr);
-	xswa.event_mask = KeyPressMask | ButtonPressMask | VisibilityChangeMask;
-
+	xswa.event_mask = KeyPressMask | ButtonPressMask |
+#ifdef MOUSEMOTION
+                          MotionNotify |
+#endif
+                          VisibilityChangeMask;
 #ifdef DEBUG
 #define WIDTH WidthOfScreen(scr) - 100
 #define HEIGHT HeightOfScreen(scr) - 100
@@ -896,12 +952,16 @@ main(argc, argv)
 #else
 #define WIDTH (inwindow? WidthOfScreen(scr)/2 : WidthOfScreen(scr))
 #define HEIGHT (inwindow? HeightOfScreen(scr)/2 : HeightOfScreen(scr))
-#define CWMASK ((inwindow? 0 : CWOverrideRedirect) | CWBackPixel | CWEventMask)
+#define CWMASK (((inwindow || (inroot && nolock))? 0 : CWOverrideRedirect) | CWBackPixel | CWEventMask)
 #endif
-
-	win[screen] = XCreateWindow(dsp, root[screen], 0, 0, WIDTH, HEIGHT, 0,
+       if (inroot) {
+            win[screen] = root[screen];
+            XChangeWindowAttributes(dsp, win[screen], CWBackPixel, &xswa);
+        } else {
+	    win[screen] = XCreateWindow(dsp, root[screen], 0, 0, WIDTH, HEIGHT, 0,
 				 CopyFromParent, InputOutput, CopyFromParent,
 				    CWMASK, &xswa);
+        }
 
 #ifdef DEBUG
 	{
@@ -968,7 +1028,7 @@ main(argc, argv)
 	XSetScreenSaver(dsp, 0, 0, 0, 0);	/* disable screen saver */
     }
 #ifndef DEBUG
-    if (!inwindow)
+    if (!inwindow && !(inroot && nolock))
       GrabKeyboardAndMouse();
 #endif
 
