@@ -10,6 +10,8 @@
  * Revision History:
  *
  * Changes of David Bagley <bagleyd@source.asset.com>
+ * 18-Nov-94: Modified for QNX 4.2 w/ Metrolink X server from Brian Campbell
+ *            <brianc@qnx.com>.
  * 11-Jul-94: added Bool flag: inwindow, which tells xlock to run in a
  *            window from Greg Bowering <greg@cs.adelaide.edu.au>
  * 11-Jul-94: patch for Solaris SYR4 from Chris P. Ross <cross@eng.umd.edu>
@@ -27,7 +29,6 @@
  *
  */
 
-#include <stdlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
@@ -35,10 +36,16 @@
 #define MAXSCREENS 3
 #define NUMCOLORS 64
 
+#ifndef min
+#define min(a,b) ((a)<(b)?(a):(b))
+#endif
+
+#if defined VMS || defined __QNX__
 #ifdef VMS
 typedef unsigned long u_long;
 typedef unsigned short u_short;
 typedef unsigned char u_char;
+#endif
 #define M_PI    3.14159265358979323846
 #define M_PI_2  1.57079632679489661923
 #define M_PI_4  0.78539816339744830962
@@ -49,11 +56,20 @@ typedef unsigned char u_char;
 #include <sys/security.h>
 #include <prot.h>
 #endif 
+
+#if !defined (news1800) && !defined (sun386)
+#include <stdlib.h>
+#ifndef apollo
+#include <unistd.h>
+#endif
+#endif
+
  
 typedef struct {
     GC          gc;		/* graphics context for animation */
     int         npixels;	/* number of valid entries in pixels */
     u_long      pixels[NUMCOLORS];	/* pixel values in the colormap */
+    u_long      bgcol, fgcol ;  /* background and foreground pixel values */
 }           perscreen;
 
 extern perscreen Scr[MAXSCREENS];
@@ -86,17 +102,21 @@ extern Bool allowaccess;
 extern Bool echokeys;
 extern Bool verbose;
 extern Bool inwindow;
+extern Bool inroot;
 extern Bool timeelapsed;
 extern void (*callback) ();
 extern void (*init) ();
 
 extern void GetResources();
 extern void hsbramp();
+#ifdef __STDC__
+extern void error(char *, ...);
+#else
 extern void error();
+#endif
 extern long seconds();
 extern void usage();
 
-
 /* System V Release 4 redefinitions of BSD functions and structures */
 #if defined(SYSV) || defined(SVR4)
 
@@ -131,11 +151,21 @@ struct passwd {
 #include <poll.h>
 #endif
 #endif
+#ifdef HPUX_SECURE_PASSWD
+#define seteuid(_eu) setresuid(-1,_eu,-1)
+#define passwd s_passwd
+#define getpwnam(_s) getspwnam(_s)
+#define getpwuid(_u) getspwuid(_u)
+#define srandom(a) srand48(a)
+#define random() lrand48()
+#define MAXRAND (2147483648.0)
+#else
 #ifndef _SGI_SOURCE
 #define srandom(a) srand(a)
 #define random() rand()
 #endif
 #define MAXRAND (32767.0)
+#endif
 
 #else
 
